/***************************************************************************\
 *                                     Math2HTML                            *
 *                                     ------------------------------------ *
 *                                     Copyright  1995 Janne Saarela/CERN  *
 \**************************************************************************/

/*
 * $Log$
 */

#ifndef MATHS_H
#define MATHS_H

#include "node.h"
#include "list.h"
#include "rtti.h"

// All classes presented in this file correspond to Latex mathematics
// primitives. They are being instantiated in the parser (parse.y)

class Sum : public Node
{
public:
    DECLARE_RTTI (Sum);

    Sum (Node *pSubscript, Node *pSupscript);
    ~Sum ();

    virtual void	convert_html3 (void);
    virtual void	convert_12083 (void);
};

class Integral : public Node
{
public:
    DECLARE_RTTI (Integral);

    Integral (Node *pSubscript, Node *pSupscript);
    ~Integral ();

    virtual void	convert_html3 (void);
    virtual void	convert_12083 (void);
};

class Fraction : public Node
{
public:
    DECLARE_RTTI (Fraction);

    Fraction (Node *pOver,
	      Node *pUnder,
	      Node *pSubscript,
	      Node *pSuperscript);
    ~Fraction ();

    virtual void	convert_html3 (void);
    virtual void	convert_12083 (void);

private:
    Node	*m_pOver;
    Node	*m_pUnder;
};

class Sequence : public Node
{
public:
    DECLARE_RTTI (Sequence);

    Sequence (List *pOptargs,
	      char *text,
	      Node *pSubscript,
	      Node *pSupscript);
    Sequence (List *pOptargs,
	      char *text,
	      Node *pSubscript,
	      Node *pSupscript,
	      List *pParameters);
    ~Sequence ();

    virtual void	convert_html3 (void);
    virtual void	convert_12083 (void);
    
private:
    char	*m_pszText;
    List	*m_pOptargs;
    List	*m_pParameters;
};

class Plain : public Node
{
public:
    DECLARE_RTTI (Plain);

    Plain (char *text, Node *pSubscript, Node *pSupscript);
    ~Plain ();

    virtual void	convert_html3 (void);
    virtual void	convert_12083 (void);

    char		*text (void);

private:
    char	*m_pszText;
};

class Group : public Node
{
public:
    DECLARE_RTTI (Group);

    Group (List *pNodes);
    ~Group ();

    virtual void	convert_html3 (void);
    virtual void	convert_12083 (void);
    
private:
    List		*m_pNodes;
};

class Table : public Node
{
public:
    DECLARE_RTTI (Table);

    Table (List *pSpecifiers, List *pEntries);
    ~Table ();

    virtual void	convert_html3 (void);
    virtual void	convert_12083 (void);

protected:
    List		*m_pEntries;
    List		*m_pSpecifiers;
};

class EqnTable : public Table
{
public:
    DECLARE_RTTI (EqnTable);

    EqnTable (List *pEntries);
    EqnTable (List *pSpecifiers, List *pEntries);
    ~EqnTable ();

    virtual void	convert_html3 (void);
    virtual void	convert_12083 (void);
};

class Root : public Node
{
public:
    DECLARE_RTTI (Root);

    Root (List *pDegree, Node *pContents);
    ~Root ();

    virtual void	convert_html3 (void);
    virtual void	convert_12083 (void);

private:
    List		*m_pDegree;
    Node		*m_pContents;
};

class WidthSpecifier : public Node
{
    friend class Table;
public:
    DECLARE_RTTI (WidthSpecifier);

    WidthSpecifier (char cType, List *pDecls);
    ~WidthSpecifier ();

private:
    char	m_cType;
    List	*m_pDecls;
};

class ColSpecifier : public Node
{
    friend class ColList;
    friend class Table;
public:
    DECLARE_RTTI (ColSpecifier);

    ColSpecifier (List *pPredecl,
		  char cAlignment,
		  WidthSpecifier *pWidth,
		  List *pPostdecl,
		  Node *pMisc);
    ~ColSpecifier ();

    int		hasVerticalBar (void);

private:
    List		*m_pPredecl;
    List		*m_pPostdecl;
    WidthSpecifier	*m_pWidth;
    char		m_cAlignment;
    Node		*m_pMisc;
};
    
class BeginMaths : public Node
{
    DECLARE_RTTI (BeginMaths);

    virtual void	convert_html3 (void) { strcat (pResult, "<math>"); }
    virtual void	convert_12083 (void) { strcat (pResult, "<formula>"); }
};

class EndMaths : public Node
{
    DECLARE_RTTI (EndMaths);

    virtual void	convert_html3 (void) { strcat (pResult, "</math>\n"); }
    virtual void	convert_12083 (void) { strcat (pResult, "</formula>\n"); }
};

class BeginTable : public Node
{
    DECLARE_RTTI (BeginTable);

    // the table attribute is to be determined by Table::convert_html3
    virtual void	convert_html3 (void) { strcat (pResult, "<table"); }
    // the array attribute is to be determined by Table::convert_12083
    virtual void	convert_12083 (void) { strcat (pResult, "<array"); }
};

class BeginEqnTable : public Node
{
    DECLARE_RTTI (BeginEqnTable);

    virtual void	convert_html3 (void) { strcat (pResult, "<table>"); }
    virtual void	convert_12083 (void) { strcat (pResult, "<array>"); }
};

class EndTable : public Node
{
    DECLARE_RTTI (EndTable);

    virtual void	convert_html3 (void) { strcat (pResult, "\n</table>\n"); }
    virtual void	convert_12083 (void) { strcat (pResult, "\n</array>\n"); }
};

class BarElement : public Node
{
    DECLARE_RTTI (BarElement);

    virtual void	convert_html3 (void) { ; }
    virtual void	convert_12083 (void) { ; }
};

class Left : public Node
{
    DECLARE_RTTI (Left);

    virtual void	convert_html3 (void) { ; }
    virtual void	convert_12083 (void) { ; }
};

class Right : public Node
{
    DECLARE_RTTI (Right);

    virtual void	convert_html3 (void) { ; }
    virtual void	convert_12083 (void) { ; }
};


#endif
